#!/usr/bin/env python
import os
import sys
import re
import string

import libpythonc.common
from libpythonc.support import logging


log = logging.getLogger(__name__)


def npath(*args):
    "returns the normalised absolute version of args joined into a path"
    return os.path.normpath(os.path.abspath(os.path.expanduser(os.path.join(*args))))


def parse_args(args):
    parser = libpythonc.common.StandardParser()
    parser.add_argument("-c", "--config", default=npath("report.txt"), help="makefile generated config file")
    parser.add_argument("module", help="output module")
    parser.add_argument("sources", nargs="+", help="module sources")
    options = parser.parse_args(args)
    config = parse_config_file("report.txt")
    return options


class SplitQuoted(object):
    _wordchars_re = _squote_re = _dquote_re = None

    def _init_regex(self):
        if self.__class__._wordchars_re is None:
            self.__class__._wordchars_re = re.compile(r'[^\\\'\"%s ]*' % string.whitespace)
            self.__class__._squote_re = re.compile(r"'(?:[^'\\]|\\.)*'")
            self.__class__._dquote_re = re.compile(r'"(?:[^"\\]|\\.)*"')


    def __call__(self, s):
        """Split a string up according to Unix shell-like rules for quotes and
        backslashes.  In short: words are delimited by spaces, as long as those
        spaces are not escaped by a backslash, or inside a quoted string.
        Single and double quotes are equivalent, and the quote characters can
        be backslash-escaped.  The backslash is stripped from any two-character
        escape sequence, leaving only the escaped character.  The quote
        characters are stripped from any quoted string.  Returns a list of
        words.
        """
        self._init_regex()


        # This is a nice algorithm for splitting up a single string, since it
        # doesn't require character-by-character examination.  It was a little
        # bit of a brain-bender to get it working right, though...

        s = string.strip(s)
        words = []
        pos = 0

        while s:
            m = self._wordchars_re.match(s, pos)
            end = m.end()
            if end == len(s):
                words.append(s[:end])
                break

            if s[end] in string.whitespace: # unescaped, unquoted whitespace: now
                words.append(s[:end])       # we definitely have a word delimiter
                s = string.lstrip(s[end:])
                pos = 0

            elif s[end] == '\\':            # preserve whatever is being escaped;
                                            # will become part of the current word
                s = s[:end] + s[end+1:]
                pos = end+1

            else:
                if s[end] == "'":           # slurp singly-quoted string
                    m = self._squote_re.match(s, end)
                elif s[end] == '"':         # slurp doubly-quoted string
                    m = self._dquote_re.match(s, end)
                else:
                    raise RuntimeError, \
                          "this can't happen (bad char '%c')" % s[end]

                if m is None:
                    raise ValueError, \
                          "bad string (mismatched %s quotes?)" % s[end]

                (beg, end) = m.span()
                s = s[:beg] + s[beg+1:end-1] + s[end:]
                pos = m.end() - 2

            if pos >= len(s):
                words.append(s)
                break

        return words


split_quoted = SplitQuoted()


def parse_config_file(config_file):
    config = {}

    for line in open(config_file, "rU"):
        index = line.find("=")
        key = line[:index].strip()
        value = line[index+1:].strip()
        config[key] = split_quoted(value)

    return config


def compile(config_or_config_file):
    config = config_or_config_file
    if isinstance(config_or_config_file, basestring):
        config = parse_config_file("report.txt")


if __name__ == "__main__":

    options = parse_args(sys.argv[1:])
    log.debug("loading config file from '%s'" % options.config)
    
    print parse_config_file(options.config)
